/*#######################################################################################
Dump-Unterprogramme zur Ausgabe eines Speicherdumps
Test der im EEPROM abgelegten IP-Adresse 

Copyright (C) 2005 Joachim Brke

#######################################################################################*/

#include "main.h"
#include "dump.h"
#include "OWIcrc.h"

extern void checkEEPROMip (void);

//***********************************************************************************************
// IP-Adresse ist jetzt einstellbar	jb 26.07.2005
// 
void checkEEPROMip(void)
{
	int i;			// Laufvariable
	unsigned char Feld[5];	// temporres Feld, um CRC-Unterpram nutzen zu knnen
	for (i=0; i<5; i++)		// alle fnf Bytes (IP mit CRC-Summe) ins RAM kopieren
		Feld[i]=eeprom_read_byte((char *)IPoffset+i);		// IP und CRC aus dem EEPROM lesen
	if (!OWI_ComputeCRCn(Feld,5))							// Wenn CRC ok,
		for (i=0; i<4; i++)	MYIP[i]=Feld[i];			//  die 4 Bytes von MYIP berschreiben
		
	for (i=0; i<256; i++)							// fr alle Bytes
		eeprom_write_byte((char *)i+768,i&0xff);	// Muster schreiben
}
// Ende
//***********************************************************************************************



//****************************************************************************
// Ausgeben eines Speicherdumps auf der seriellen Schnittstelle
//
//	bytes enthlt die Anzahl der Bytes, die ausgegeben werden sollen
//	*Datenlesen zeigt auf die Funktion, die die nchsten 16 Byte zur Verfgung stellt
//
void dump (int bytes, void (*Datenlesen)(int))
{
	int j=0;					// Adresszhler
	unsigned char zeile[80];	// Puffer fr die Ausgabezeile
	
	Adresszeile (zeile,0);		// Adrezeile anlegen
	printf("%s",zeile);		// und ausgeben

	for (j=0; j<bytes; j+=16) {				// insgesamt Bytes/16 Datenzeilen ausgeben
		Datenzeile (zeile,0,j,*Datenlesen);	// Datenzeile erstellen
		printf("%s",zeile);					// und ausgeben		
	}
	printf("\n");							// Leerzeile zum Abschlu
}

//****************************************************************************
// Lschen des Ausgabepuffers
//
//	*Zeile zeigt auf den Buffer
//  Der Buffer wird mit Leerzeichen initialisiert und mit Zeilen- und Stringende versehen
//  Zwischen den beiden 8-Byte-Blcken wird ein '-' ausgegeben
//
void Initzeile (char *zeile, int zindex)
{
	unsigned char i;
	for(i=0;i<79;i++) zeile[zindex+i]=' ';	// Zeile vorbereiten
	zeile[zindex+32] = '-';					// in der Mitte "-" als Trenner drucken
	zeile[zindex+77] = 0x0d;					// CR setzen
	zeile[zindex+78] = 0x0a;					// LF setzen
	zeile[zindex+79] = 0;						// Stringende setzen
}

//****************************************************************************
// Ausgeben eines Bytes in Hexadezimaldarstellung in einen Buffer
//	*buffer zeigt auf den Buffer
//  xx enthlt das Datenbyte, das ausgegeben werden soll
//	aa enthlt die Position im Buffer
//
void hex_zeile (char *buffer,char xx, int aa)
{
	unsigned char x;
	x = xx >> 4;
	buffer[aa++] = x>9 ? x+'A'-10 : x+'0';
	x = xx & 0x0f;
	buffer[aa] = x>9 ? x+'A'-10 : x+'0'; 
}

//****************************************************************************
// Bereitstellen der Datenzeile
//
//	*Zeile zeigt auf den Buffer
//
void Datenzeile (char *zeile, int zindex, int adr, void (*Datenlesen)(int))
{
	unsigned char k,l;
	Initzeile (zeile,zindex);
	
	(*Datenlesen) (adr);				// 16 Byte einlesen

	hex_zeile(zeile,adr/256, zindex);		// "Adresse :" an den Zeilenanfang setzen
	hex_zeile(zeile,adr&0xff,zindex+2);
	zeile[zindex+5] = ':';					// Doppelpunkt hinter der Adresse setzen

	for(k=0;k<=7;k++)  hex_zeile(zeile,DATA9346[k],zindex+k*3+8);	// 16 2-stellige HEX-Zeichen drucken
	for(k=8;k<=15;k++) hex_zeile(zeile,DATA9346[k],zindex+k*3+10);

	for(k=0; k<=15; k++) {			// 16 ASCII-Zeichen drucken
		l = DATA9346[k];
		if ((l<32)|(l>126)) l='.';	// nicht druckbare Zeichen als "." ausgeben
		if (l=='<') l='.';			// '<' wird als '.' dargestellt!
		zeile[zindex+k+61]=l;
	} 
}

//****************************************************************************
// Bereitstellen der Adresszeile
//
//	*Zeile zeigt auf den Anfang des Buffers
//	zindex zeigt auf die Position, ab der geschrieben werden soll
//
void Adresszeile (char *zeile, int zindex)
{
	unsigned char k;
	Initzeile (zeile,zindex);
	for(k=0;k<=7;k++)  hex_zeile(zeile,k,zindex+k*3+8);	// Adrezeile anlegen
	for(k=8;k<=15;k++) hex_zeile(zeile,k,zindex+k*3+10);
	zeile[zindex+67] = 0x0d;		// CR setzen (eine Leerzeile nach der Adresszeile)
	zeile[zindex+68] = 0x0a;		// LF setzen
}

//****************************************************************************
// Lesen von 16 Byte fr den 93LC46-Speicherdump
//
void read9346 (int dummy)
{
	unsigned char k;
	for (k=0; k<=15; k++) DATA9346[k]=0;		// Datenfeld initialisieren 
		
	for (k=0; k<=127; k++) {					// 128 bit aus dem EEPROM in das Datenfeld lesen
		WriteRTL (CR9346,(EEM1 |EECS ));		// Takt = 0	
		WriteRTL (CR9346,(EEM1 |EECS |EESK));	// Takt = 1
		DATA9346[(k>>3)^1] |= (ReadRTL(CR9346)&EEDO)<<(~k&7);// Bit im Datenfeld speichern
	}
	// MAC-Adresse der Netzwerkkarte in das Feld "mymac[]" bernehmen
	if (dummy==0)	for (k=0; k<=5; k++)	mymac[k]=DATA9346[k+4];
}

//****************************************************************************
// Lesen von 16 Byte fr den EEPROM-Speicherdump
//
void readEEPROM (int Adresse)
{
	unsigned char k;
	for (k=0; k<=15; k++) DATA9346[k]=eeprom_read_byte((char *)Adresse+k);			
}
